﻿ ; utf-8 BOM
;******************************************************************************
; Product:  MpStart version 1.2.0 (2024-05-07)
; Purpose:  Start MediaPortal from a shortcut or BAT file.
; Author:   CyberSimian (contact via fora at: www.team-mediaportal.com)
;
; o  This Support Pack is for use with MCE RC6 remote controls. These remote
;    controls have a HID device called "Microsoft eHome Infrared Transceiver"
;    in Windows "Device Manager".
;
; o  The Support Pack is not suitable for remote controls that send keyboard
;    shortcuts, such as the Ortek VRC-1100 and Hama 00052451. See instead the
;    "Centarea HID Support Pack" for a script that works with those remote
;    controls.
;
; o  The Support Pack provides the following capabilities:
;    -  Starts MediaPortal version 1 from the Windows desktop. It does this
;       more reliably than the MediaPortal "MpTray" utility (which on some
;       systems fails to work first time after a reboot).
;    -  Allows MediaPortal to start, even when the preceding invocation of
;       MediaPortal did not close correctly. This problem occurs occasionally
;       on some systems, and prevents MediaPortal starting (whether by using
;       the START button on the remote control, or by clicking a shortcut on
;       the desktop).
;    -  Allows MediaPortal to be started by using the colour buttons on the
;       remote control. The colour buttons can be defined with different
;       MediaPortal options, for example: to start MediaPortal using a skin
;       that is different from the default skin.
;
; o  The script works with:
;    -  32-bit MediaPortal version 1 running on 32-bit Windows
;    -  32-bit MediaPortal version 1 running on 64-bit Windows
;    -  64-bit MediaPortal version 1 running on 64-bit Windows
;
; o  The script uses AutoHotKey version 2 syntax. If you already have version
;    1 installed on your system, you can download and install version 2, as
;    the two versions can co-exist on the same system. When a script is run,
;    the appropriate AutoHotKey version is used. The AutoHotKey tool can be
;    downloaded from here:
;      https://www.autohotkey.com/
;
; o  The script uses its own functions for accessing the HID data, but these
;    functions were derived from the "AhkHid" suite of functions created by
;    "TheGood". The file "AhkHid.ahk" can be downloaded from here:
;      https://github.com/jleb/AHKHID
;    The AhkHid functions themselves are discussed here:
;      https://www.autohotkey.com/board/topic/38015-ahkhid-an-ahk-implementation-of-the-hid-functions/
;    Note that the file "AhkHid.ahk" uses AutoHotKey version 1 syntax.
;
; o  To install the MpStart Support Pack:
;    -  Download "MpStart.zip" and unzip it into a dedicated folder. This
;       folder can be placed anywhere on your system, but do not use a folder
;       located in the "Team MediaPortal" folder tree.
;    -  Run the file "install.bat". This copies the AHK files the appropriate
;       folder in the MP folder tree, and places a startup shortcut in your
;       startup folder.
;    -  If you use "Run as Administrator", the startup shortcut is placed
;       in your "All Users" startup folder. If that cannot be accessed, the
;       shortcut is placed in the current user's startup folder.
;    -  To stop using the support pack, run "uninstall.bat".
;    -  You do not need to reboot your system after running "install.bat" or
;       "uninstall.bat", as the changes take effect immediately.
;
;******************************************************************************
; INITIALISATION (AUTO-EXECUTE SECTION; EXECUTED ONCE-ONLY)
;******************************************************************************

; Environment
  #Requires Autohotkey v2.0
  #Warn All, MsgBox
  #SingleInstance Off
  Persistent(false)

; Detect hidden windows (allows application to be restored when minimised)
  DetectHiddenWindows(true)

; Check for EXIT action (used when uninstalling support pack)
  If  (A_Args.Length = 1)
  And (A_Args[1] = ":EXIT:")
  {
;-- Force the "MpTrap" script to exit
    If (WinExist("MpTrap"))
      WinClose("MpTrap", ,1)             ; wait for up to 1 second
    ExitApp(0)
  }

; Get invocation args as a single string
; Note: we cannot recreate exactly the arg list specified by the user, due
; to the way that double-quote characters are processed by AutoHotKey. So we
; add double-quote characters around each arg, as in most cases this will be
; interpreted correctly by the application invoked. Example:
;   arg list:   /skin="Titan"
;   becomes:    "/skin=Titan"
  arg_list := ""
  For (arg in A_Args)
    arg_list := arg_list " `"" arg "`""
; MsgBox("Arg list: [" arg_list "]")   ; DEBUG

; Get application definitions
  DefineApp()

; Run the application
  rc := StartApp(arg_list)

  ExitApp(rc)


;******************************************************************************
; FUNCTIONS
;******************************************************************************
; DefineApp() -- Define the location of the application to be started.
; StartApp()  -- Start the application.
;******************************************************************************

DefineApp()
{
; Return codes
  Global RC_APP_STARTED, RC_APP_RESTORED , RC_APP_RUNNING
  Global RC_APP_REMNANT, RC_APP_NOT_FOUND, RC_BUTTON_NOT_APPLIC
; Application values
  Global app_32_spec, app_64_spec, app_log_spec
  Global app_file   , app_name
  Global start_button, blue_button, green_button, red_button, yellow_button

; Define the application drive
  app_drv := "C:"

; Define the locations of the application folders and files
  app_32_path   := app_drv "\Program Files (x86)\Team MediaPortal\MediaPortal"
  app_64_path   := app_drv "\Program Files\Team MediaPortal\MediaPortal"
  app_data_path := app_drv "\ProgramData\Team MediaPortal\MediaPortal"
  app_name      := "MediaPortal"

; Subsidiary definitions constructed from the above
  app_file      := app_name ".exe"
  app_32_spec   := app_32_path "\" app_file
  app_64_spec   := app_64_path "\" app_file
  app_log_spec  := app_data_path "\Log\MpStart.log"

; Define the MP window group
; Note: we need the trailing "-" in "title_prefix" to avoid matching windows
; that do not belong to "MediaPortal.exe" (e.g. MP Watchdog, Windows Explorer).
; We also need to exclude windows that have "Configuration" in the name, in
; order to exclude the following windows:
;   "MediaPortal - Configuration"
;   "MediaPortal - TV Server Configuration"
  title_prefix := "MediaPortal - "
  GroupAdd("mp_group", title_prefix, , "Configuration")

; Get settings from INI file
  start_button  := IniRead("MpStart.ini", "buttons", "start" , "")
  red_button    := IniRead("MpStart.ini", "buttons", "red"   , "")
  green_button  := IniRead("MpStart.ini", "buttons", "green" , "")
  yellow_button := IniRead("MpStart.ini", "buttons", "yellow", "")
  blue_button   := IniRead("MpStart.ini", "buttons", "blue"  , "")

; Define return codes
  RC_APP_STARTED   := 1
  RC_APP_RESTORED  := 2
  RC_APP_RUNNING   := 3
  RC_APP_REMNANT   := 4
  RC_APP_NOT_FOUND := 5
  RC_BUTTON_NOT_APPLIC := 6

  Return
}

;******************************************************************************

StartApp(arg_list)
{
; Return codes
  Global RC_APP_STARTED, RC_APP_RESTORED , RC_APP_RUNNING
  Global RC_APP_REMNANT, RC_APP_NOT_FOUND, RC_BUTTON_NOT_APPLIC
; Application values
  Global app_32_spec, app_64_spec, app_log_spec
  Global app_file   , app_name

; MsgBox("Arglist: [" arg_list "]")    ; DEBUG

; Detect hidden windows (allows application to be restored when minimised)
  DetectHiddenWindows(true)

; Check if application is running
  If (!WinExist("ahk_group mp_group")) ; application has no windows?
  {
;-- Check if the application did not stop completely when closed previously;
;-- this happens occasionally, and prevents the application restarting.
    pid := ProcessExist(app_file)
    If (pid != 0)                      ; application is still running?
    {
;---- Write line to log file.
      timestamp := FormatTime("", "yyyy-MM-dd HH:mm:ss")
      text := "Terminating leftover remnant of " app_name "."
      FileAppend(timestamp "  " text "`n", app_log_spec)
;---- Force application to close
;---- Note: "ProcessWaitClose()" is sometimes unable to force closure of the
;---- application, so we use "taskkill.exe" with the "/f" option instead.
      RunWait("taskkill.exe /pid " pid " /f", , "hide")
;---- Wait for taskkill to complete (value is milliseconds)
      Sleep(500)
;---- Check if successful
      pid := ProcessExist(pid)
      If (pid != 0)
      {
        text := "Unable to terminate leftover remnant; reason: [" A_LastError "]."
        FileAppend(timestamp "  " text "`n", app_log_spec)
        MsgBox(text)
        Return(RC_APP_REMNANT)
      }
    }

;-- Start the application (start 64-bit if both 32-bit and 64-bit exist)
;   timestamp := FormatTime("", "yyyy-MM-dd HH:mm:ss")
;   text := "Starting " app_name "."
;   FileAppend(timestamp "  " text "`n", app_log_spec)
    If (app_64_spec != "") And (FileExist(app_64_spec))
    {
      Run("`"" app_64_spec "`" " arg_list)
      Return(RC_APP_STARTED)
    }
    Else If (app_32_spec != "") And (FileExist(app_32_spec))
    {
      Run("`"" app_32_spec "`" " arg_list)
      Return(RC_APP_STARTED)
    }
    Else
    {
      MsgBox("`"" app_file "`" not found.")
      Return(RC_APP_NOT_FOUND)
    }
  }

; Check if application is minimised to the system tray, or...
; application is running but does not have the focus
  Else If (!WinActive("ahk_group mp_group"))
  {
    WinActivate("ahk_group mp_group")
    WinShow("ahk_group mp_group")
    Return(RC_APP_RESTORED)
  }

; Application is running and has the focus
  Else
  {
    Return(RC_APP_RUNNING)
  }

}

;******************************************************************************
