﻿ ; utf-8 BOM
;******************************************************************************
; Product:  MpStart version 1.2.0 (2024-05-07)
; Purpose:  Trap the START and colour buttons on the MCE RC6 remote control.
; Author:   CyberSimian (contact via fora at: www.team-mediaportal.com)
;
; Notes:
; 1.  Refer to "MpStart.ahk" and "MpStart.txt" for further details.
;******************************************************************************

;******************************************************************************
; INITIALISATION (AUTO-EXECUTE SECTION; EXECUTED ONCE-ONLY)
;******************************************************************************

; Environment
  #Requires Autohotkey v2.0
  #Warn All, MsgBox
  #SingleInstance Force
  Persistent(true)
  script_name := Rtrim(Rtrim(A_ScriptName,"ahk"),".")

; Get application definitions
  DefineApp()

; Set title match-mode (1=match start, 2=match substring, 3=match exactly)
  SetTitleMatchMode(1)

; Detect hidden windows (allows application to be restored when minimised)
  DetectHiddenWindows(true)

; Create a hidden placeholder window
  myGui := Gui()
  myGui.OnEvent("Close", GuiClose)
  myGui.Title := script_name
  myGui.Show("Hide")
  hwin := WinExist(script_name)

; Trap HID msgs in all windows ("RIDEV_INPUTSINK")
  usage_page := 65468
  usage      := 136
; flags      := RIDEV_INPUTSINK
  flags      := 0x00000100
  rc := Register(usage_page,usage,hwin,flags) ; (never return here if error)

; Intercept WM_INPUT messages
  OnMessage(0x00FF, InputMsg)

  Return


;******************************************************************************

; Action when placeholder window is closed
GuiClose(*)
{
  ExitApp(0)
}


;******************************************************************************
; FUNCTIONS
;******************************************************************************
; DefineApp() -- Define the location of the application to be started.
; StartApp()  -- Start the application.
; Register()  -- Register the HID device.
; InputMsg()  -- Receive and process a HID message.
; GetButton() -- Get the button code from the HID message.
;
; Notes:
; 1.  The "Register()" and "GetButton()" functions have been derived from
;     the AHKHID suite of functions written by "TheGood".
; 2.  "MsgBox" affects the active window, which causes "InputMsg()" to
;     malfunction. So "MsgBox" can be used only if followed by "ExitApp",
;     or subsequent processing is not dependent on the active window.
;******************************************************************************

DefineApp()
{
; Return codes
  Global RC_APP_STARTED, RC_APP_RESTORED , RC_APP_RUNNING
  Global RC_APP_REMNANT, RC_APP_NOT_FOUND, RC_BUTTON_NOT_APPLIC
; Application values
  Global app_32_spec, app_64_spec, app_log_spec
  Global app_file   , app_name
  Global start_button, blue_button, green_button, red_button, yellow_button

; Define the application drive
  app_drv := "C:"

; Define the locations of the application folders and files
  app_32_path   := app_drv "\Program Files (x86)\Team MediaPortal\MediaPortal"
  app_64_path   := app_drv "\Program Files\Team MediaPortal\MediaPortal"
  app_data_path := app_drv "\ProgramData\Team MediaPortal\MediaPortal"
  app_name      := "MediaPortal"

; Subsidiary definitions constructed from the above
  app_file      := app_name ".exe"
  app_32_spec   := app_32_path "\" app_file
  app_64_spec   := app_64_path "\" app_file
  app_log_spec  := app_data_path "\Log\MpStart.log"

; Define the MP window group
; Note: we need the trailing "-" in "title_prefix" to avoid matching windows
; that do not belong to "MediaPortal.exe" (e.g. MP Watchdog, Windows Explorer).
; We also need to exclude windows that have "Configuration" in the name, in
; order to exclude the following windows:
;   "MediaPortal - Configuration"
;   "MediaPortal - TV Server Configuration"
  title_prefix := "MediaPortal - "
  GroupAdd("mp_group", title_prefix, , "Configuration")

; Get settings from INI file
  start_button  := IniRead("MpStart.ini", "buttons", "start" , "")
  red_button    := IniRead("MpStart.ini", "buttons", "red"   , "")
  green_button  := IniRead("MpStart.ini", "buttons", "green" , "")
  yellow_button := IniRead("MpStart.ini", "buttons", "yellow", "")
  blue_button   := IniRead("MpStart.ini", "buttons", "blue"  , "")

; Define return codes
  RC_APP_STARTED   := 1
  RC_APP_RESTORED  := 2
  RC_APP_RUNNING   := 3
  RC_APP_REMNANT   := 4
  RC_APP_NOT_FOUND := 5
  RC_BUTTON_NOT_APPLIC := 6

  Return
}

;******************************************************************************

StartApp(arg_list)
{
; Return codes
  Global RC_APP_STARTED, RC_APP_RESTORED , RC_APP_RUNNING
  Global RC_APP_REMNANT, RC_APP_NOT_FOUND, RC_BUTTON_NOT_APPLIC
; Application values
  Global app_32_spec, app_64_spec, app_log_spec
  Global app_file   , app_name

; MsgBox("Arglist: [" arg_list "]")    ; DEBUG

; Detect hidden windows (allows application to be restored when minimised)
  DetectHiddenWindows(true)

; Check if application is running
  If (!WinExist("ahk_group mp_group")) ; application has no windows?
  {
;-- Check if the application did not stop completely when closed previously;
;-- this happens occasionally, and prevents the application restarting.
    pid := ProcessExist(app_file)
    If (pid != 0)                      ; application is still running?
    {
;---- Write line to log file.
      timestamp := FormatTime("", "yyyy-MM-dd HH:mm:ss")
      text := "Terminating leftover remnant of " app_name "."
      FileAppend(timestamp "  " text "`n", app_log_spec)
;---- Force application to close
;---- Note: "ProcessWaitClose()" is sometimes unable to force closure of the
;---- application, so we use "taskkill.exe" with the "/f" option instead.
      RunWait("taskkill.exe /pid " pid " /f", , "hide")
;---- Wait for taskkill to complete (value is milliseconds)
      Sleep(500)
;---- Check if successful
      pid := ProcessExist(pid)
      If (pid != 0)
      {
        text := "Unable to terminate leftover remnant; reason: [" A_LastError "]."
        FileAppend(timestamp "  " text "`n", app_log_spec)
        MsgBox(text)
        Return(RC_APP_REMNANT)
      }
    }

;-- Start the application (start 64-bit if both 32-bit and 64-bit exist)
;   timestamp := FormatTime("", "yyyy-MM-dd HH:mm:ss")
;   text := "Starting " app_name "."
;   FileAppend(timestamp "  " text "`n", app_log_spec)
    If (app_64_spec != "") And (FileExist(app_64_spec))
    {
      Run("`"" app_64_spec "`" " arg_list)
      Return(RC_APP_STARTED)
    }
    Else If (app_32_spec != "") And (FileExist(app_32_spec))
    {
      Run("`"" app_32_spec "`" " arg_list)
      Return(RC_APP_STARTED)
    }
    Else
    {
      MsgBox("`"" app_file "`" not found.")
      Return(RC_APP_NOT_FOUND)
    }
  }

; Check if application is minimised to the system tray, or...
; application is running but does not have the focus
  Else If (!WinActive("ahk_group mp_group"))
  {
    WinActivate("ahk_group mp_group")
    WinShow("ahk_group mp_group")
    Return(RC_APP_RESTORED)
  }

; Application is running and has the focus
  Else
  {
    Return(RC_APP_RUNNING)
  }

}

;******************************************************************************

Register(UsagePage, Usage, Handle, Flags)
{
; Get buffer for structure
  data_buf := Buffer((8+A_PtrSize), 0)

; Check if window handle needs to be null (RIDEV_REMOVE or RIDEV_EXCLUDE)
  Handle := ((Flags & 0x00000001) Or (Flags & 0x00000010)) ? 0 : Handle

; Build structure
  NumPut("UShort", UsagePage, data_buf, 0)
  NumPut("UShort", Usage    , data_buf, 2)
  NumPut("UInt"  , Flags    , data_buf, 4)
  NumPut("Uint"  , Handle   , data_buf, 8)

; Register the device
  ok := DllCall("RegisterRawInputDevices", "Ptr", data_buf, "UInt", 1, "UInt", 8+A_PtrSize)
  If (!ok)
  {
    MsgBox("RegisterRawInputDevices() error;`nReason: [" A_LastError "].")
    ExitApp(101)
  }

  Return 0
}

;******************************************************************************

InputMsg(wparam, lparam, msg, hwnd)
{
; Return codes
  Global RC_APP_STARTED, RC_APP_RESTORED , RC_APP_RUNNING
  Global RC_APP_REMNANT, RC_APP_NOT_FOUND, RC_BUTTON_NOT_APPLIC
; Application-specific values
  Global app_32_spec, app_64_spec, app_log_spec
  Global app_file   , app_name
  Global start_button, blue_button, green_button, red_button, yellow_button

; "rc" must be static to propagate its value from one invocation to the next.
  Static rc := 0

; Get the button code
  Critical("On")                       ; prevent possible ERROR_INVALID_HANDLE
  button := GetButton(lparam)          ; (never return here if error)
  Critical("Off")

; Check for button up (all buttons)
  If (button = 768)                    ; button up (same for all buttons)
  {
    If (rc = RC_APP_RUNNING)           ; button-down was allowed?
    Or (rc = RC_BUTTON_NOT_APPLIC)     ; or button not applicable?
      Return                           ; allow other windows to see button-up
    Else
      Return 0                         ; prevent other windows seeing button-up
  }

; Set parameter list according to button pressed
  If (button = 781)                    ; START button (button down)
    parm_list := start_button
  Else If (button = 859)               ; RED button (button down)
    parm_list := red_button
  Else If (button = 860)               ; GREEN button (button down)
    parm_list := green_button
  Else If (button = 861)               ; YELLOW button (button down)
    parm_list := yellow_button
  Else If (button = 862)               ; BLUE button (button down)
    parm_list := blue_button
  Else                                 ; some other button-down
  {
    rc := RC_BUTTON_NOT_APPLIC         ; set rc for next invocation
    Return                             ; allow other windows to see button
  }

; Button is START/RED/GREEN/YELLOW/BLUE
  rc := StartApp(parm_list)            ; start the application
  If (rc = RC_APP_RUNNING)             ; app already running and has focus?
    Return                             ; allow other windows to see button
  Else
    Return 0                           ; prevent other windows seeing button

}

;******************************************************************************

GetButton(Handle)
{
; Get a buffer for the raw HID input (overestimate length for safety)
; Using a fixed-length buffer avoids the need to use "DllCall()" to invoke
; the "GetRawInputData()" function to determine the length of the data.
  hdr_len  := 2*A_PtrSize + 8
  buf_len  := 48                       ; overestimated (bytes)
  data_buf := Buffer(buf_len)

; Get the raw input                                              RID_INPUT
  data_len := DllCall("GetRawInputData", "UInt", Handle, "UInt", 0x10000003, "Ptr", data_buf, "UInt*", &buf_len, "UInt", hdr_len)
  If (data_len = -1)
  {
    MsgBox("GetRawInputData() error;`nReason: [" A_LastError "].")
    ExitApp(102)
  }
  If (data_len < hdr_len+10)           ; 10 == HID entry hdr + one HID entry
  {
    MsgBox("GetRawInputData() error;`nReason: Unexpected data length: [" data_len "].")
    ExitApp(103)
  }

; Calculate the button code (skip over the header)
;   Start=781; Red=859; Green=860; Yellow=861; Blue=862
  button := NumGet(data_buf, hdr_len+8, "uchar")*256 + NumGet(data_buf, hdr_len+9, "uchar")
; MsgBox("Button code: [" button "]")  ; DEBUG: display the button code

  Return button
}

;******************************************************************************
